import argparse
import sys
import tables
from generator import get_prediction_generators
import h5py
from keras.utils.io_utils import HDF5Matrix
import os
from keras.optimizers import Adam
from keras.models import Sequential, load_model
from PIL import Image
import cv2

parser = argparse.ArgumentParser(description='Running Model Training')
parser.add_argument('--test_data', type=str, required=True)
parser.add_argument('--trained_model', type=str, required=True)
parser.add_argument('--out_dir', type=str, required=True)

parsed_data = parser.parse_args(sys.argv[1:])

print(parsed_data.test_data)
print(parsed_data.trained_model)
print(parsed_data.out_dir)
#exit()

model = load_model(parsed_data.trained_model)

print ("model: ", model)
opt = Adam(lr=0.001, decay=1e-6)
model.compile(loss='binary_crossentropy', optimizer=opt, metrics = ['accuracy'])

h5_folder = parsed_data.test_data
directories = sorted([d for d in os.listdir(h5_folder) if os.path.isdir(os.path.join(h5_folder, d)) and not d.startswith(".")])
print(directories)

for dir_name in directories :
	out_dir = os.path.join(parsed_data.out_dir, dir_name)
	if not os.path.isdir(out_dir):
		os.mkdir(out_dir)
	print(os.listdir(os.path.join(h5_folder, dir_name)))
	for file_name in os.listdir(os.path.join(h5_folder, dir_name)):
		if file_name.endswith('h5'):
			temp_out_dir = os.path.join(out_dir, 'v'+file_name.split('v')[-1].split('.')[0])
			if not os.path.isdir(temp_out_dir):
				os.mkdir(temp_out_dir)

			print ("Prediction started on File: ", file_name)

			data_file_opened = tables.open_file(os.path.join(h5_folder, dir_name, file_name), "r")
			image_generator = get_prediction_generators(data_file_opened,
                        		                            batch_size=data_file_opened.root.data.shape[0],
                                        		            patch_shape=(5, 128, 128, 1)
                                                       		   )

			predict_out = model.predict_generator(image_generator, steps=1, verbose=1)
			print(predict_out.shape)

			f = h5py.File(os.path.join(temp_out_dir, file_name.split('v')[0]+'predicted.h5'), 'w')
			f.create_dataset('data', data=predict_out)

			print ("Prediction completed on File: ", file_name)
	print ("Prediction completed on DIR: ", dir_name)

print ("Hurry!! Prediction Completed")

